package ru.dbotthepony.kstarbound.io.json.builder

private fun Int.toBool() = if (this == 0) null else this > 0

/**
 * Указывает, что для данного класса можно автоматически создать [BuilderAdapter] для всех его свойств,
 * которые не указаны как [JsonIgnoreProperty]
 *
 * @see JsonIgnoreProperty
 * @see JsonPropertyConfig
 */
@Target(AnnotationTarget.CLASS)
@Retention(AnnotationRetention.RUNTIME)
annotation class JsonBuilder(
	/**
	 * @see BuilderAdapter.Builder.extraPropertiesAreFatal
	 */
	val extraPropertiesAreFatal: Boolean = false,

	/**
	 * @see BuilderAdapter.Builder.ignoreKey
	 */
	val ignoreKeys: Array<String> = [],

	/**
	 * 0 = null
	 * -1 = false
	 * 1 = true
	 *
	 * @see BuilderAdapter.Builder.logMisses
	 */
	val logMisses: Int = 0,

	val includeSuperclassProperties: Boolean = true,
)

val JsonBuilder.realLogMisses get() = logMisses.toBool()

/**
 * Заставляет указанное свойство быть проигнорированным при автоматическом создании [BuilderAdapter]
 */
@Target(AnnotationTarget.PROPERTY)
@Retention(AnnotationRetention.RUNTIME)
annotation class JsonIgnoreProperty

/**
 * Выставляет флаги данному свойству при автоматическом создании [BuilderAdapter]
 *
 * @see BuilderAdapter.Builder.add
 */
@Target(AnnotationTarget.PROPERTY)
@Retention(AnnotationRetention.RUNTIME)
annotation class JsonPropertyConfig(
	val isFlat: Boolean = false,

	val mustBePresent: Int = 0,
)

val JsonPropertyConfig.realMustBePresent get() = mustBePresent.toBool()

/**
 * Указывает, что для данного класса можно автоматически создать [FactoryAdapter]
 */
@Target(AnnotationTarget.CLASS)
@Retention(AnnotationRetention.RUNTIME)
annotation class JsonFactory(
	/**
	 * @see FactoryAdapter.Builder.storesJson
	 */
	val storesJson: Boolean = false,

	/**
	 * @see FactoryAdapter.Builder.logMisses
	 */
	val logMisses: Boolean = true,

	/**
	 * @see FactoryAdapter.Builder.inputAsList
	 * @see FactoryAdapter.Builder.inputAsMap
	 */
	val asList: Boolean = false
)
